@@ -11,6 +11,7 @@ from bfit.backend.fitdata import fitdata
 from bfit.gui.popup_prepare_data import popup_prepare_data
 from bfit.backend.entry_color_set import on_focusout, on_entry_click
 import bfit.backend.colors as colors
+from bfit.global_variables import KEYVARS
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
@@ -43,12 +44,14 @@ class fetch_files(object):
             entry_asym_type: combobox for asym calc and draw type
             entry_run: entry to put in run number string
             fet_entry_frame: frame of fetch tab
+            filter_opt: StringVar, holds state of filter radio buttons
             listbox_history: listbox for run input history
             max_number_fetched: max number of files you can fetch
             omit_state: BooleanVar, if true set omit all final incomplete scans
             run: StringVar input to fetch runs.
             runmode_label: display run mode
             runmode: display run mode list of strings
+            text_filter: Text box for filter commands input
             year: IntVar of year to fetch runs from 
     """
     
@@ -150,12 +153,28 @@ class fetch_files(object):
         dataline_frame.bind("<Configure>", self.config_canvas) # bind resize to alter scrollable region
         self.data_canvas.bind("<Configure>", self.config_dataline_frame) # bind resize to change size of contained frame
         
+        
+        
         # Frame to hold everything on the right ------------------------------
         bigright_frame = ttk.Frame(fetch_data_tab, pad=5)
         
+        # asymmetry calculation
+        style_frame = ttk.Labelframe(bigright_frame, text='Asymmetry Calculation', \
+                pad=5)
+        self.asym_type = StringVar()
+        self.asym_type.set('')
+        self.entry_asym_type = ttk.Combobox(style_frame, \
+                textvariable=self.asym_type, state='readonly', \
+                width=20)
+        self.entry_asym_type['values'] = ()
+        
+        style_frame.grid(column=0, row=0, sticky=(W, N, E))
+        style_frame.grid_columnconfigure(0, weight=1)
+        self.entry_asym_type.grid(column=0, row=0, sticky=(N, E, W), padx=10)
+        
         # Frame for group set options ----------------------------------------
         right_frame = ttk.Labelframe(bigright_frame, \
-                text='Operations on Active Items', pad=30)
+                text='Operations on Active Items', pad=15)
         
         check_remove = ttk.Button(right_frame, text='Remove', \
                 command=self.remove_all, pad=5)
@@ -250,7 +269,7 @@ class fetch_files(object):
         runmode_label_frame.grid(column=2, row=0, sticky=(N, W, E, S), pady=5, padx=5)
         self.runmode_label.grid(column=0, row=0, sticky=(N, W, E))
         
-        bigright_frame.grid(column=2, row=1, sticky=(N, E))
+        bigright_frame.grid(column=2, row=1, rowspan=2, sticky='new')
         
         self.data_canvas.grid(column=0, row=1, sticky=(E, W, S, N), padx=5, pady=5)
         yscrollbar.grid(column=1, row=1, sticky=(W, S, N), pady=5)
@@ -260,14 +279,47 @@ class fetch_files(object):
         check_fit_box.grid(         column=2, row=0, sticky=(N))
         check_res_box.grid(         column=3, row=0, sticky=(N)) 
         
-        right_frame.grid(           column=0, row=0, sticky=(N, E, W))
+        right_frame.grid(           column=0, row=1, sticky=(N, E, W), pady=5)
         r = 0
         right_checkbox_frame.grid(  column=0, row=r, sticky=(N), columnspan=2); r+= 1
         check_toggle_button.grid(   column=0, row=r, sticky=(N, E, W), columnspan=2, pady=1, padx=5); r+= 1
         check_draw.grid(            column=0, row=r, sticky=(N, W, E), pady=5, padx=5);
         check_remove.grid(          column=1, row=r, sticky=(N, E, W), pady=5, padx=5); r+= 1
         frame_scan.grid(            column=0, row=r, sticky=(N, E, W), pady=5, padx=5, columnspan=2); r+= 1        
-        bigright_frame.grid(        rowspan=2, sticky=(N, E, W))
+        
+        # filtering -----------------------------------------------------------
+        frame_filter = ttk.Labelframe(bigright_frame, text='Filter Runs', pad=5)
+        button_filter_instr = ttk.Button(frame_filter, text='Instructions', 
+                                         command=self.show_filter_instructions)
+        self.text_filter = Text(frame_filter, width=35, height=5, state='normal', 
+                                wrap='none')
+        
+        frame_filter_button = ttk.Frame(frame_filter, pad=5)
+        button_filter = ttk.Button(frame_filter_button, text='Filter', 
+                                         command=self.filter_runs)
+        self.filter_opt = StringVar()
+        self.filter_opt.set('activate')
+        radio_filter_activate = ttk.Radiobutton(frame_filter_button, 
+                                       variable=self.filter_opt, 
+                                       value='activate',
+                                       text='By activatation',
+                                       )
+        radio_filter_remove = ttk.Radiobutton(frame_filter_button, 
+                                       variable=self.filter_opt, 
+                                       value='remove',
+                                       text='By removal',
+                                       )
+        
+        button_filter.grid(column=0, row=0, rowspan=2, sticky='w', padx=5)
+        radio_filter_activate.grid(column=1, row=0, sticky='w', padx=5)
+        radio_filter_remove.grid(column=1, row=1, sticky='w', padx=5)
+        
+        # final grid
+        frame_filter.grid(column=0, row=2, sticky='new', pady=5)
+        r = 0
+        button_filter_instr.grid(column=0, row=r, sticky='new', pady=5); r+= 1
+        self.text_filter.grid(column=0, row=r, sticky='new', pady=5); r+= 1
+        frame_filter_button.grid(column=0, row=r, sticky='new', pady=5); r+= 1
         
         # resizing
         fetch_data_tab.grid_columnconfigure(0, weight=1)        # main area
@@ -280,20 +332,6 @@ class fetch_files(object):
         self.data_canvas.grid_columnconfigure(0, weight=1)    # fetch frame 
         self.data_canvas.grid_rowconfigure(0, weight=1)
             
-        # asymmetry calculation
-        style_frame = ttk.Labelframe(bigright_frame, text='Asymmetry Calculation', \
-                pad=5)
-        self.asym_type = StringVar()
-        self.asym_type.set('')
-        self.entry_asym_type = ttk.Combobox(style_frame, \
-                textvariable=self.asym_type, state='readonly', \
-                width=20)
-        self.entry_asym_type['values'] = ()
-        
-        style_frame.grid(column=0, row=1, sticky=(W, N, E))
-        style_frame.grid_columnconfigure(0, weight=1)
-        self.entry_asym_type.grid(column=0, row=0, sticky=(N, E, W), padx=10)
-        
         # passing
         self.entry_run = entry_run
         self.entry_year = entry_year
@@ -319,6 +357,33 @@ class fetch_files(object):
         except Exception:
             pass
         
+    # ======================================================================= #
+    def _check_condition(self, data, string):
+        """Check whether a run satisfies the condition given as a string"""
+        
+        # determine what variable is being tested
+        keyvars = np.array(list(KEYVARS.keys()))
+        keylens = [len(k) for k in keyvars]
+        keyvars = keyvars[np.argsort(keylens)[::-1]]
+        
+        found = False
+        for key in keyvars:
+            if key in string:
+                found = True
+                break
+        
+        if not found:
+            msg = 'Variable not recognized in "%s"' % string
+            messagebox.showerror('Filter', msg)
+            raise RuntimeError(msg)
+        
+        # replace key with value
+        val, err = data.get_values(KEYVARS[key])
+        string = string.replace(key, str(val))
+        
+        # evaluate
+        return eval(string)
+    
     # ======================================================================= #
     def _do_check_all(self, state, var, box):
         """
@@ -346,7 +411,7 @@ class fetch_files(object):
             # run check for other items
             if hasattr(dline, 'do_'+var):
                 getattr(dline, 'do_'+var)()
-
+    
     # ======================================================================= #
     def canvas_scroll(self, event):
         """Scroll canvas with files selected."""
@@ -466,6 +531,35 @@ class fetch_files(object):
                              omit=omit)
         self.logger.debug('Success.')
         
+    # ======================================================================= #
+    def filter_runs(self):
+        """Remove or select runs based on filter conditions"""
+        
+        # parse input string
+        string = self.text_filter.get('1.0', END)
+        lines = [l for l in string.split('\n') if l]
+        
+        # iterate on fetched data
+        data_keep = []
+        
+        for k, d in self.bfit.data.items():
+            
+            # check conditions
+            satisfy = all([self._check_condition(d, line) for line in lines])
+            if satisfy:
+                data_keep.append(k)
+            
+        # do filtering by selection
+        if self.filter_opt.get() == 'activate':
+            for k, d in self.bfit.data.items():
+                d.check_state.set(k in data_keep)
+                
+        elif self.filter_opt.get() == 'remove':
+            keys = tuple(self.data_lines.keys())
+            for k in keys:
+                if k not in data_keep:
+                    self.data_lines[k].degrid()
+    
     # ======================================================================= #
     def get_data(self):
         """Split data into parts, and assign to dictionary."""
@@ -742,6 +836,12 @@ class fetch_files(object):
         elif focus_id == self.check_all_box:
             self.logger.debug('Focus is: check all box')
             self.draw_all()
+            
+        # filter
+        elif focus_id == self.text_filter:
+            self.logger.debug('Focus is: text_filter')
+            self.filter_runs()
+            
         else:
             pass
 
@@ -811,6 +911,32 @@ class fetch_files(object):
         
         self.bfit.set_nbm(mode)
     
+    # ======================================================================= #
+    def show_filter_instructions(self):
+        """Display popup with instructions on filtering"""
+        
+        # instructions
+        msg = 'Write one boolean expression per line using the the below '+\
+              'variables. \n\n'+\
+              'Runs which satisfy all expressions are kept/activated\n\n'
+              
+        # variables
+        keys = sorted(KEYVARS.keys())
+        values = [KEYVARS[k] for k in keys]
+        max_keylen = max(tuple(map(len, keys)))
+        
+        lines = [k.ljust(max_keylen+2) + v for k, v in zip(keys, values)]
+              
+        msg += '   ' + ('\n   '.join(lines))
+        
+        # example
+        ex = 'ex: "10 < BIAS < 15"'
+        msg += '\n\n' + ex
+        
+        # show message
+        messagebox.showinfo(title = 'How to filter runs',
+                            message = msg)
+            
     # ======================================================================= #
     def string2run(self, string):
         """Parse string, return list of run numbers"""
